;
; ####### Beispielprogramm fr die CPU6502 am NKC            #######
; ####### Zum Uebersetzen wird der TASM verwendet.           #######
; ####### Hier soll kurz gezeigt werden, welche Vektoren     #######
; ####### minimal gesetzt werden mssen und wie man eigenen  #######
; ####### Programmcode startet.                              #######
;

; ####### defines fuer 6502 #######

#define     VEC_NMI     $fffa         ; wird bei NMI angesprungen
#define     VEC_RESET   $fffc         ; wird bei RESET angesprungen
#define     VEC_IRQ     $fffe         ; wird bei maskierbarem Interrupt angesprungen
#define     ZPDATA      $00           ; 6502 Zeropage


; ####### defines fuer NKC #######

#define     EPROM_START $e000         ; EPROM auf Platz 8 der ROA64
#define     IOBASE      $c000         ; Jumpereinstellung auf CPU6502


; ####### defines fuer NKC KEY #######

#define     KEYD        IOBASE + $68
#define     KEYS        IOBASE + $69


; ####### defines fuer NKC GDP64 #######

#define     GDP         IOBASE + $70
#define     PAGE        IOBASE + $60


; ####### Einsprungadressen setzen #######
; Diese liegen im EPROM an den 6502-spezifischen Adressen und enthalten die
; Adresse, die bei RESET, NMI und IRQ angesprungen werden soll.
; Das sind die letzten 6 Bytes im EPROM auf Platz 8 der ROA64.

        .org VEC_NMI
        .word GO_NMI

        .org VEC_RESET
        .word GO_RESET

        .org VEC_IRQ
        .word GO_IRQ


; ####### Programmcode #######
; Das Programm soll hier am Beginn des EPROMS auf Platz 8 der ROA64 starten.

        .org EPROM_START

; Zuerst der Behandlungscode fr den NMI. Die Interruptservicerouting tut nichts
; und kehrt zum Hauptprogramm zurck. Hier kann der Code fr den Timer eingebaut
; werden.

GO_NMI
        rti

; Hier die Routine fr den maskierbaren Interrupt. Auch sie kehrt einfach zum Hauptprogramm zurck.

GO_IRQ
        rti

; Hier beginnt das Hauptprogramm. Der folgende Code wird nach jedem Reset, inklusive 
; Einschalten des Computers ausgefhrt.

GO_RESET
        cld         ; Binaermode
        ldx #$ff    ; Stack
        txs         ; initialisieren


; GDP64 initialisieren
        jsr gdpinit

; Zeichen in KEY bei Systemstart loeschen
        jsr clearkey



        lda #$41        ; Zeichen A
        sta GDP         ; Auf Bildschirm ausgeben
        jsr gdpwait

        lda #$42        ; Zeichen B
        sta GDP         ; Auf Bildschirm ausgeben
        jsr gdpwait

        lda #$43        ; Zeichen C
        sta GDP         ; Auf Bildschirm ausgeben
        jsr gdpwait

echo
        jsr getkey      ; Zeichen von Tastatur
        sta GDP         ; auf Bildschirm ausgeben
        jmp echo        ; Endlosschleife



; ##### Hilfsroutinen fuer die GDP64 #####

; ##### initgdp initialisiert die GDP64 minimal #####
gdpinit
        pha             ; AKKU auf Steck retten

        lda #0          ; Seite 0 verwenden
        sta PAGE

        jsr gdpwait

        lda #6          ; Bildschirm
        sta GDP         ; loeschen

        jsr gdpwait

        lda #3          ; PEN down
        sta GDP+1       ; (schreiben)

        pla             ; Akku zurueckholen
        rts


; ##### gdpwait wartet bis GDP64 Befehl ausgefuehrt hat #####
gdpwait
        pha             ; AKKU auf Steck retten
        lda #4          ; warten
gdpwait1                ; bis
        bit GDP         ; GDP64
        beq gdpwait1    ; fertig ist
        pla             ; Akku zurueckholen
        rts


; ##### Hilfsroutinen fuer die KEY Abfrage #####

; ##### clearkey loescht im Puffer wartenden Tastendruck #####
clearkey
        pha             ; Akku auf Stack retten
        lda KEYS        ; Tastenstatus loeschen
        pla             ; Akku zurueckholen
        rts

; ##### waitkey wartet auf einen Tastendruck #####
waitkey
        pha             ; Akku auf Stack retten
        lda #$80
wk1     bit KEYD        ; Wurde Taste gedrueckt?
        bne wk1         ; sonst warten
        lda KEYD        ; Tastencode holen
        lda KEYS        ; Tastenstatus loeschen
        pla             ; Akku zurueckholen
        rts


; ##### getkey wartet auf Tastendruck und liefert Tastencode im Akku #####
getkey
        lda #$80
        bit KEYD        ; Wurde Taste gedrueckt?
        bne getkey      ; sonst warten in Schleife
        lda KEYD        ; Tastencode holen
        pha
        lda KEYS        ; Tastenstatus loeschen
        pla
        rts

.end
